@@ -1,5 +1,18 @@
-autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, check.stochastic = TRUE, raftery.options = list(), crash.retry=1, plots = TRUE, thin.sample = TRUE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200), modules=c(""), factories=c(""), thin = 1, monitor.deviance = FALSE, monitor.pd = FALSE, monitor.pd.i = FALSE, monitor.popt = FALSE, keep.jags.files=FALSE, tempdir=TRUE, method=if(.Platform$OS.type=='unix' & .Platform$GUI!="AQUA") 'interruptible' else 'simple'){
+autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No monitored variables supplied"), data=NA, n.chains=2, inits = replicate(n.chains, NA), startburnin = 5000, startsample = 10000, psrf.target = 1.05, normalise.mcmc = TRUE, check.stochastic = TRUE, raftery.options = list(), crash.retry=1, plots = TRUE, thin.sample = TRUE, jags = findjags(), silent.jags = FALSE, interactive=TRUE, max.time=Inf, adaptive=list(type="burnin", length=200), modules=c(""), factories=c(""), thin = 1, monitor.deviance = FALSE, monitor.pd = FALSE, monitor.pd.i = FALSE, monitor.popt = FALSE, keep.jags.files=FALSE, tempdir=TRUE, method=if(.Platform$OS.type=='unix' & .Platform$GUI!="AQUA" & Sys.info()['user']!='nobody') 'interruptible' else 'simple'){
+	
+	if(any(tolower(monitor)=='dic')){
+		dic <- TRUE
+		monitor.deviance <- TRUE
+		monitor.pd <- TRUE
+		monitor.popt <- TRUE
+	}else{
+		dic <- FALSE
+	}
 
+	if(any(tolower(monitor)=='deviance')){
+		monitor.deviance <- TRUE
+	}
+	
 	if(any(c(monitor.deviance, monitor.pd, monitor.popt, monitor.pd.i))) modules <- c(modules, "dic")
 	modules <- unique(modules)
 	modules <- as.character(na.omit(modules[modules!=""]))
@@ -85,22 +98,22 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	
 	newlines <- if(silent.jags) "\n" else "\n\n"
 	
-	cat("\nAuto-run JAGS",newlines,"Running a pilot chain...\n",sep="")
+	swcat("\nAuto-run JAGS",newlines,"Running a pilot chain...\n",sep="")
 	pre.time <- Sys.time()
 	
 	
 	pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 	if(any(pilot=="Unable to load coda files")){
-		cat("An error occured during the simulation\n\n")
+		swcat("An error occured during the simulation\n\n")
 		return(c("Error", "An error occured during the simulation"))
 	}	
 	
 	if(any(is.na(pilot))){
-		cat("An error occured during the simulation\n\n")
+		swcat("An error occured during the simulation\n\n")
 		return(c("Error", "An error occured during the simulation"))
 	}
 	if(class(pilot)=="character"){
-		cat("An error occured during the simulation\n\n")
+		swcat("An error occured during the simulation\n\n")
 		return(c("Error", "An error occured during the simulation"))
 	}
 	
@@ -108,10 +121,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		repeat{
 			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 			if(time.taken > max.time | crash.retry==0){
-				cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+				swcat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
 				return(c("Error", "The simulation was aborted due to crashes"))
 			}
-			cat("\nThe simulation crashed; retrying...",newlines,sep="")			
+			swcat("\nThe simulation crashed; retrying...",newlines,sep="")			
 			crash.retry <- crash.retry - 1
 			oldpilot <- pilot
 			pilot <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = inits, burnin=startburnin, sample=startsample, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
@@ -119,22 +132,23 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			if(!any(names(pilot)=="crash.end")) break
 		}
 	}
-	
+
 	firsttimetaken = time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
-	cat("\n")
+	swcat("\n")
 	
 	otheroutputs <- vector("list")
-	if(monitor.deviance) otheroutputs <- c(otheroutputs, deviance=list(pilot$deviance))
 	if(monitor.pd) otheroutputs <- c(otheroutputs, pd=list(pilot$pd))
 	if(monitor.pd.i) otheroutputs <- c(otheroutputs, pd.i=list(pilot$pd.i))
 	if(monitor.popt) otheroutputs <- c(otheroutputs, popt=list(pilot$popt))
-	
+	if(dic) otheroutputs <- c(otheroutputs, dic=list(pilot$dic))
+
 	final.mcmc <- pilot$mcmc
 	
-	cat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
+	swcat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
+	
 	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
 	if(class(success)=="try-error"){
-		cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
+		swcat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 		return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 	}
 	
@@ -169,7 +183,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				mpsrfstring <- paste(" (multi-variate psrf = ", round(convergence$mpsrf, digits=3), ")", sep="")
 			}
 			
-			cat("The Gelman-Rubin statistic was above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", n.iters, " iterations", mpsrfstring, ".  This may indicate poor convergence.\n", sep="")
+			swcat("The Gelman-Rubin statistic was above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", n.iters, " iterations", mpsrfstring, ".  This may indicate poor convergence.\n", sep="")
 			updatesthrown <- updatesthrown + startsample
 			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 			stop <- time.taken > max.time
@@ -181,11 +195,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			}
 			if(stop){
 				
-				cat("Calculating autocorrelation and summary statistics...\n")
+				swcat("Calculating autocorrelation and summary statistics...\n")
 				
 				suppressWarnings(success <- try(autocorr <- safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 				if(class(success)=="try-error"){
-					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 					return(c("Error", "An error occured while calculating the autocorrelation"))
 				}
 
@@ -193,12 +207,17 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 				options(show.error.messages = TRUE)
 				
-				cat("Returning UNCONVERGED simulation results\n\n")
+				options(show.error.messages = FALSE)
+				suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+				thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+				options(show.error.messages = TRUE)	
 				
-				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+				swcat("Returning UNCONVERGED simulation results\n\n")
+				
+				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 			}else{
 				finishconv <- FALSE
-				cat("Extending the simulation to attempt to improve convergence...\n")
+				swcat("Extending the simulation to attempt to improve convergence...\n")
 				thrownaway <- 0
 			}
 			
@@ -217,11 +236,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=neededupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 					
 					if(any(pilot=="Unable to load coda files")){
-						cat("\nThere was an error in the second simulation, possibly due to bad initial values or Random Number Seed values obtained from the first simulation.  You could try using fewer chains.")
+						swcat("\nThere was an error in the second simulation, possibly due to bad initial values or Random Number Seed values obtained from the first simulation.  You could try using fewer chains.")
 					}
 					
 					if(any(additional=="Unable to load coda files")){
-						cat("An error occured during the simulation\n\n")
+						swcat("An error occured during the simulation\n\n")
 						return(c("Error", "An error occured during the simulation"))
 					}
 					if(any(names(additional)=="crash.end")){
@@ -229,10 +248,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 							time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 							
 							if(time.taken > max.time | crash.retry==0){
-								cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+								swcat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
 								return(c("Error", "The simulation was aborted due to crashes"))
 							}
-							cat("\nThe simulation crashed; retrying...",newlines,sep="")
+							swcat("\nThe simulation crashed; retrying...",newlines,sep="")
 							crash.retry <- crash.retry - 1
 							oldadd <- additional
 							additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = oldadditional$end.state, burnin=burnadapt, sample=(neededupdates), adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
@@ -243,13 +262,13 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					
 					pilot <- additional
 					
-					final.mcmc <- window(combine.mcmc(list(final.mcmc, additional$mcmc)), collapse.chains=FALSE,  start=(((niter(final.mcmc)+niter(additional$mcmc))-startsample)+1))
+					suppressWarnings(final.mcmc <- window(combine.mcmc(list(final.mcmc, additional$mcmc)), collapse.chains=FALSE,  start=(((niter(final.mcmc)+niter(additional$mcmc))-startsample)+1)))
 					
-					cat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
+					swcat("Calculating the Gelman-Rubin statistic for ", nvar(final.mcmc), " variables....\n", sep="")
 					thrownaway <- thrownaway+neededupdates
 					suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), transform=FALSE, autoburnin=TRUE), silent=TRUE))
 					if(class(success)=="try-error"){
-						cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
+						swcat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 						return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 					}
 					
@@ -272,10 +291,10 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 						}	
 					}
 					
-					if(monitor.deviance) otheroutputs$deviance <- additional$deviance
 					if(monitor.pd) otheroutputs$pd <- additional$pd
 					if(monitor.pd.i) otheroutputs$pd.i <- additional$pd.i
 					if(monitor.popt) otheroutputs$popt <- additional$popt
+					if(dic) otheroutputs$dic <- additional$dic
 					
 					if(class(convergence$mpsrf)!="numeric"){
 						mpsrfstring <- " (Unable to calculate the multi-variate psrf)"
@@ -284,7 +303,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 					}
 					
 					if(unconverged > 0){
-						cat("The Gelman-Rubin statistic was still above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", updatesdone + neededupdates, " iterations", mpsrfstring, ".\n", sep="")
+						swcat("The Gelman-Rubin statistic was still above ", psrf.target, " for ", unconverged, " parameter", if(unconverged>1) "s", " after ", updatesdone + neededupdates, " iterations", mpsrfstring, ".\n", sep="")
 						
 						stop <- time.taken > max.time
 						if(interactive){
@@ -294,11 +313,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 						}
 						if(stop){
 							
-							cat("Calculating autocorrelation and summary statistics...\n")
+							swcat("Calculating autocorrelation and summary statistics...\n")
 							
 							suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 							if(class(success)=="try-error"){
-								cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+								swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 								return(c("Error", "An error occured while calculating the autocorrelation"))
 							}
 
@@ -306,29 +325,52 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 							suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 							options(show.error.messages = TRUE)
 							
-							cat("Returning UNCONVERGED simulation results\n\n")
+							options(show.error.messages = FALSE)
+							suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+							thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+							options(show.error.messages = TRUE)	
 							
-							return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+							swcat("Returning UNCONVERGED simulation results\n\n")
+							
+							return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 						}else{
-							cat("Extending the simulation to attempt to improve convergence...\n")
+							swcat("Extending the simulation to attempt to improve convergence...\n")
 							updatesdone <- updatesdone + neededupdates
 							updatesthrown <- updatesthrown + neededupdates
 						}
 						
 					}else{
-						cat(paste("The Gelman-Rubin statistic is now below ", psrf.target, " for all parameters\n", sep=""))
+						swcat(paste("The Gelman-Rubin statistic is now below ", psrf.target, " for all parameters\n", sep=""))
 						finishconv <- TRUE
 					}
 				}else{
-					cat("Maximum time limit exceeded; chains still unconverged.  Try restarting the simulation using the end state values of the chains provided\n")
-					cat("Returning UNCONVERGED simulation results\n\n")
-					return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+					swcat("Maximum time limit exceeded; chains still unconverged.  Try restarting the simulation using the end state values of the chains provided\n")
+					swcat("Calculating autocorrelation and summary statistics...\n")
+					
+					suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
+					if(class(success)=="try-error"){
+						swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
+						return(c("Error", "An error occured while calculating the autocorrelation"))
+					}
+
+					options(show.error.messages = FALSE)
+					suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
+					options(show.error.messages = TRUE)
+					
+					options(show.error.messages = FALSE)
+					suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+					thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+					options(show.error.messages = TRUE)	
+					
+				
+					swcat("Returning UNCONVERGED simulation results\n\n")
+					return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=NA, pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 				}
 			
 			}
 	
 		}else{
-			cat(paste("The Gelman-Rubin statistic is below ", psrf.target, " for all parameters\n", sep=""))
+			swcat(paste("The Gelman-Rubin statistic is below ", psrf.target, " for all parameters\n", sep=""))
 			updatesthrown <- 0
 		}
 	#}else{
@@ -337,7 +379,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	
 	
 	
-	cat("\nCalculating the necessary sample length based on the Raftery and Lewis's diagnostic...\n")
+	swcat("\nCalculating the necessary sample length based on the Raftery and Lewis's diagnostic...\n")
 			
 	success <- try({
 	raftery.args$data <- normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)
@@ -366,7 +408,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	dependancethreshold <- 3
 	
 	if(any(dependance > dependancethreshold) & killautocorr==FALSE){
-				cat("IMPORTANT:  The sample size of monitored node(s) '", paste(dimnames(dependance)[[1]][apply(dependance, 1, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse="' & '"), "' have a high autocorrelation dependance in chain(s) ", paste(seq(1, n.chains)[apply(dependance, 2, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse= " & "), ".  Re-running the model with a different formulation or better initial values may help to reduce autocorrelation.\n", sep="")
+				swcat("IMPORTANT:  The sample size of monitored node(s) '", paste(dimnames(dependance)[[1]][apply(dependance, 1, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse="' & '"), "' have a high autocorrelation dependance in chain(s) ", paste(seq(1, n.chains)[apply(dependance, 2, function(x) if(any(x>dependancethreshold)) return(TRUE) else return(FALSE))], collapse= " & "), ".  Re-running the model with a different formulation or better initial values may help to reduce autocorrelation.\n", sep="")
 	}
 	
 	#moreupdates <- ((max(sample+(max(0, burnin-startburnin)))-(startsample*n.chains))/n.chains)*1.05
@@ -376,15 +418,15 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	if(moreupdates > 0){
 	
 		success <- try({
-			testmatrix <- matrix(nrow=(max(sample)/thin)*n.chains, ncol=length(newmonitor))
-			if(monitor.deviance) testmatrix2 <- matrix(nrow=(max(sample)/thin)*n.chains, ncol=length(pilot$deviance)) else testmatrix2 <- NA
-			if(monitor.popt) testmatrix3 <- matrix(nrow=(max(sample)/thin)*n.chains, ncol=length(pilot$popt)) else testmatrix3 <- NA
-			if(monitor.pd) testmatrix4 <- matrix(nrow=(max(sample)/thin)*n.chains, ncol=length(pilot$pd)) else testmatrix4 <- NA
+			testmatrix <- matrix(nrow=(max(sample)/thin)*n.chains, ncol=length(newmonitor)+as.numeric(monitor.deviance))
+			if(monitor.pd.i) testmatrix2 <- matrix(nrow=nrow(pilot$pd.i), ncol=1) else testmatrix2 <- NA
+			if(monitor.popt) testmatrix3 <- matrix(nrow=nrow(pilot$popt), ncol=1) else testmatrix3 <- NA
+			if(monitor.pd) testmatrix4 <- matrix(nrow=(max(sample)/thin), ncol=1) else testmatrix4 <- NA
 			rm(testmatrix, testmatrix2, testmatrix3, testmatrix4)
 			
 		})
 		if(class(success)=="try-error"){
-			cat("The model needs to be run for a further ", moreupdates, " iterations.  This would create a vector too large to be read into R.  Try re-parameterising the model to reduce autocorrelation, using the thin option to reduce autocorrelation, or monitoring less variables.  The simulation can be restarted using the chain end states provided", sep="")
+			swcat("The model needs to be run for a further ", moreupdates, " iterations.  This would create a vector too large to be read into R.  Try re-parameterising the model to reduce autocorrelation, using the thin option to reduce autocorrelation, or monitoring less variables.  The simulation can be restarted using the chain end states provided", sep="")
 			return(list(error="Unable to create vector of necessary sample size", req.samples=max(sample), end.state=pilot$end.state))
 		}
 	
@@ -392,12 +434,12 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		if(FALSE){  ###### START NOT RUN
 			## BELOW WAS FOR ADDING RAFTERY EXTENSION TO TIME LIMIT (MODIFIED AND BROKEN...)
 		if((timestring(pre.time, Sys.time(), units="s", show.units=FALSE)+(timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE))) < max.time){
-			cat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
+			swcat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
 			if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)) if(!ask("Continue with the simulation?")){
-				cat("Simulation aborted\n\n")
+				swcat("Simulation aborted\n\n")
 				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 				if(class(success)=="try-error"){
-					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 					return(c("Error", "An error occured while calculating the autocorrelation"))
 				}
 
@@ -405,19 +447,24 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 				options(show.error.messages = TRUE)
 				
-				cat("Calculating autocorrelation and summary statistics...\n")
-				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+				options(show.error.messages = FALSE)
+				suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+				thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+				options(show.error.messages = TRUE)	
+				
+				swcat("Calculating autocorrelation and summary statistics...\n")
+				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 				
 			}
 		}else{
-			cat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
+			swcat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
 			#cat("The model would need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((time.taken*moreupdates/(startsample+startburnin))), ", taking the total simulation length to over ", timestring(max.time), ".\n", sep="")  NOW NO LONGER ADDING THIS TO TIME LIMIT
 			stop <- TRUE
 			if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)) stop <- !ask("Continue with the simulation?")
 			if(stop){
 				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 				if(class(success)=="try-error"){
-					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 					return(c("Error", "An error occured while calculating the autocorrelation"))
 				}
 
@@ -425,39 +472,49 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 				options(show.error.messages = TRUE)
 				
-				cat("Simulation aborted\n\n")
-				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+				options(show.error.messages = FALSE)
+				suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+				thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+				options(show.error.messages = TRUE)	
+				
+				swcat("Simulation aborted\n\n")
+				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 			}
 		}
 		}##### END NOT RUN
 		
-		cat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
+		swcat("The model will need to be run for a further ", moreupdates, " updates.  This will take approximately ", timestring((firsttimetaken*moreupdates/(startsample+startburnin))), ".\n", sep="")
 		if(interactive & (timestring((time.taken*moreupdates/(startsample+startburnin)), units="s", show.units=FALSE)>60)){
 			time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 			if(!ask("Continue with the simulation?")){
 				
-				cat("Calculating autocorrelation and summary statistics...\n")
+				swcat("Calculating autocorrelation and summary statistics...\n")
 				suppressWarnings(success <- try(autocorr <-  safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 				if(class(success)=="try-error"){
-					cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+					swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 					return(c("Error", "An error occured while calculating the autocorrelation"))
 				}
 
 				options(show.error.messages = FALSE)
 				suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 				options(show.error.messages = TRUE)
+				options(show.error.messages = FALSE)
+				suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+				thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+				options(show.error.messages = TRUE)	
+				
 				pre.time <- Sys.time()-time.taken
-				cat("Simulation aborted\n\n")
-				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
+				swcat("Simulation aborted\n\n")
+				return(c(list(pilot.mcmc=final.mcmc, end.state=pilot$end.state, req.samples=max(sample), pilot.summary=fsummary, pilot.HPD=thpd, samples.to.conv=NA, psrf=convergence, autocorr=autocorr), otheroutputs))
 			}
 			pre.time <- Sys.time()-time.taken
 		}
 		
 		
-		cat("\n")
+		swcat("\n")
 		additional <- run.jags(data=data, model=model, monitor=monitor,  n.chains=n.chains, inits=pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 		if(any(additional=="Unable to load coda files")){
-			cat("An error occured during the simulation\n\n")
+			swcat("An error occured during the simulation\n\n")
 			return(c("Error", "An error occured during the simulation"))
 		}
 		
@@ -465,15 +522,15 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			repeat{
 				time.taken <- timestring(pre.time, Sys.time(), units="secs", show.units=FALSE)
 				if(time.taken > max.time | crash.retry==0){
-					cat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
+					swcat("The simulation exceeded the number of crashes allowed by crash.retry and so was aborted\n\n")
 					return(c("Error", "The simulation was aborted due to crashes"))
 				}
-				cat("\nThe simulation crashed; retrying...",newlines,sep="")			
+				swcat("\nThe simulation crashed; retrying...",newlines,sep="")			
 				crash.retry <- crash.retry - 1
 				oldadd <- additional
 				additional <- run.jags(data=data, model=model, monitor=monitor, n.chains=n.chains, inits = pilot$end.state, burnin=burnadapt, sample=moreupdates, adapt=adapt, silent.jags=silent.jags, plots = FALSE, check.conv=FALSE, jags = jags, psrf.target = psrf.target, normalise.mcmc=normalise.mcmc, check.stochastic = check.stochastic, modules=modules, factories=factories, thin=thin, monitor.deviance=monitor.deviance, monitor.pd=monitor.pd, monitor.pd.i=monitor.pd.i, monitor.popt=monitor.popt, keep.jags.files=keep.jags.files, tempdir=tempdir, method=method)
 				if(any(additional=="Unable to load coda files")){
-					cat("An error occured during the simulation\n\n")
+					swcat("An error occured during the simulation\n\n")
 					return(c("Error", "An error occured during the simulation"))
 				}
 				if(!any(names(additional)=="crash.end")) break
@@ -487,27 +544,26 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		additional <- pilot # for end.state at the end, pilot is output of either first sim or extended convergence sim
 	}	
 	
-	cat("Necessary sample length achieved\n")
+	swcat("Necessary sample length achieved\n")
 	
 	final.mcmc <- combine.mcmc(final.mcmc, collapse.chains=FALSE, return.samples=thin.sample)
 	
-	if(monitor.deviance) otheroutputs$deviance <- additional$deviance
 	if(monitor.pd) otheroutputs$pd <- additional$pd
 	if(monitor.pd.i) otheroutputs$pd.i <- additional$pd.i
 	if(monitor.popt) otheroutputs$popt <- additional$popt
-	
+	if(dic) otheroutputs$dic <- additional$dic
 	
 	#final.mcmc <- window(final.mcmc, start=max(burnin))
 	n.iters <- niter(final.mcmc)
 	
 	suppressWarnings(success <- try(convergence <- safe.gelman.diag(normalise.mcmcfun(final.mcmc, normalise=normalise.mcmc, warn=FALSE, check.stochastic = check.stochastic), warn=TRUE, transform=FALSE, autoburnin=TRUE), silent=TRUE))
 	if(class(success)=="try-error"){
-		cat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
+		swcat("An error occured while calculating the Gelman-Rubin statistic; aborting simulation.  Check that different chains have not been given the same starting values and random seeds.\n")
 		return(c("Error", "An error occured while calculating the Gelman-Rubin statistic"))
 	}
 	suppressWarnings(success <- try(autocorrelation <- safe.autocorr.diag(normalise.mcmcfun(final.mcmc, normalise=FALSE, warn=FALSE, check.stochastic = check.stochastic)), silent=TRUE))
 	if(class(success)=="try-error"){
-		cat("An error occured while calculating the autocorrelation; aborting simulation\n")
+		swcat("An error occured while calculating the autocorrelation; aborting simulation\n")
 		return(c("Error", "An error occured while calculating the autocorrelation"))
 	}
 	
@@ -516,6 +572,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	suppressWarnings(fsummary <- summary(combine.mcmc(final.mcmc, collapse.chains=FALSE)))
 	options(show.error.messages = TRUE)
 	
+	options(show.error.messages = FALSE)
+	suppressWarnings(thpd <- HPDinterval(combine.mcmc(final.mcmc, collapse.chains=TRUE)))
+	thpd <- cbind(lower95=thpd[,1], median=apply(combine.mcmc(final.mcmc, collapse.chains=TRUE),2,median), upper95=thpd[,2])
+	options(show.error.messages = TRUE)	
+	
 	convergence <- c(convergence, psrf.target=psrf.target)
 	class(convergence) <- "gelman.with.target"
 
@@ -542,7 +603,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	}
 	
 	if(unconverged > 0 & !killautocorr){
-		cat(paste("WARNING:  The Gelman-Rubin statistic for ", unconverged, " parameter", if(unconverged>1) "s", " are above ", psrf.target, " for the final chains", mpsrfstring, ".  The chains may have fallen out of convergence.  It is important to assess convergence manually before relying on inference from these chains.\n", sep=""))
+		swcat(paste("WARNING:  The Gelman-Rubin statistic for ", unconverged, " parameter", if(unconverged>1) "s", " are above ", psrf.target, " for the final chains", mpsrfstring, ".  The chains may have fallen out of convergence.  It is important to assess convergence manually before relying on inference from these chains.\n", sep=""))
 	}
 	
 	if(n.params==1) convergence$mpsrf <- NULL
@@ -561,12 +622,12 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 	if(killautocorr==FALSE){
 		if(!is.na(param.autocorr)){
 			if(autocorrelated > 0 & moreupdates > 0){
-				cat("REMINDER:  There was a high degree of autocorrelation for ", autocorrelated, " parameter", if(autocorrelated>1) "s", "\n", sep="")
+				swcat("REMINDER:  There was a high degree of autocorrelation for ", autocorrelated, " parameter", if(autocorrelated>1) "s", "\n", sep="")
 			}else{
 				#cat("Convergence achieved for this run\n")
 			}
 		}else{
-			cat("Autocorrelation could not be calculated for these chains\n")
+			swcat("Autocorrelation could not be calculated for these chains\n")
 		}
 		unused <- normalise.mcmcfun(final.mcmc, normalise=FALSE, warn="warning", check.stochastic = check.stochastic)  # SO THAT WARNING OF NON STOCHASTICITY IS PRINTED
 	}
@@ -575,7 +636,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		success <- try({
 		plot1 = plot2 = vector('list', length=length(varnames(final.mcmc)))
 		names(plot1) = names(plot2) <- varnames(final.mcmc)
-		thinned.mcmc <- combine.mcmc(list(final.mcmc), collapse.chains=FALSE, return.samples=1000)
+		thinned.mcmc <- combine.mcmc(list(final.mcmc), collapse.chains=FALSE, return.samples=min(1000, niter(final.mcmc)))
 
 		#startdev <- dev.list()
 
@@ -588,7 +649,7 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 			varnames(plotdata)[2] <- 'dummy' # Different name prevents warning about duplicate factors
 			plot1[[i]] <- xyplot(plotdata, layout=c(1,1), ylab="Value", xlab="Iteration")
 			class(plot1[[i]]) <- "plotindpages"
-			plot2[[i]] <- densityplot(plotdata, layout=c(1,1), ylab="Density", xlab="Value")
+			plot2[[i]] <- densityplot(plotdata, plot.points=FALSE, layout=c(1,1), ylab="Density", xlab="Value")
 			class(plot2[[i]]) <- "plotindpages"
 
 			# ...and then remove the index for the unnecessary second plot afterwards:
@@ -617,11 +678,11 @@ autorun.jags <- function(model=stop("No model supplied"), monitor = stop("No mon
 		plot1 = plot2 <- "Plots not produced when plots==FALSE"
 	}
 	
-	cat("Auto-run JAGS complete.\n\n")
-	if(killautocorr==FALSE) cat("*PLEASE NOTE:  THIS SOFTWARE IS INTENDED FOR EDUCATIONAL PURPOSES ONLY*\n*YOU SHOULD ASSESS CONVERGENCE AND AUTOCORRELATION MANUALLY BEFORE RELYING ON RESULTS PROVIDED*\n\n")
+	swcat("Auto-run JAGS complete.\n\n")
+	if(killautocorr==FALSE) swcat("*PLEASE NOTE:  THIS SOFTWARE IS INTENDED FOR EDUCATIONAL PURPOSES ONLY*\n*YOU SHOULD ASSESS CONVERGENCE AND AUTOCORRELATION MANUALLY BEFORE RELYING ON RESULTS PROVIDED*\n\n")
 	
 	
-	return(c(list(mcmc=final.mcmc, end.state=additional$end.state, req.samples=max(sample), samples.to.conv=updatesthrown, thin=thin, summary=fsummary, psrf=convergence, autocorr=autocorrelation, trace=plot1, density=plot2), otheroutputs))
+	return(c(list(mcmc=final.mcmc, end.state=additional$end.state, req.samples=max(sample), samples.to.conv=updatesthrown, thin=thin, summary=fsummary, HPD=thpd, psrf=convergence, autocorr=autocorrelation, trace=plot1, density=plot2), otheroutputs))
 }
 	
 autorun.JAGS <- autorun.jags
